@@ -5,21 +5,21 @@ import tempfile
 import unittest
 import email
 import datetime
-from textwrap import dedent # make sample configuration files more readable.
-from StringIO import StringIO
 from optparse import Values
 
 from weblog.post import Post, PostError, Author
-from weblog.template import environment
 from weblog.publish import load_posts
 from weblog.date import command_date
 from weblog.publish import command_publish
 from weblog import configuration
 from weblog.html_full_url import FullUrlHtmlParser
 from weblog.utf8_html_parser import UTF8HTMLParser
+from weblog.rfc3339 import LocalTimeTestCase
 
-def _file(string):
-    return StringIO(dedent(string))
+_DIRNAME = os.path.dirname(__file__)
+
+def _test_filename(filename):
+    return os.path.join(_DIRNAME, 'test', filename)
 
 def _default_dict(**kwargs):
     d = dict(author=Author(''),
@@ -35,7 +35,7 @@ def _default_dict(**kwargs):
 class TestSimpleLoad(unittest.TestCase):
 
     def test_load_post_list(self):
-        post_list = load_posts('test/simple/', _default_dict())
+        post_list = load_posts(_test_filename('simple'), _default_dict())
         self.assertEqual(len(post_list), 3)
         sorted_list = sorted(post_list)
         self.assertEqual(sorted_list[0].title, 'post1')
@@ -43,11 +43,11 @@ class TestSimpleLoad(unittest.TestCase):
         self.assertEqual(sorted_list[2].title, 'post3')
 
     def test_load_post_list_encoding_failure(self):
-        self.assertRaises(PostError, load_posts, 'test/encoding/',
+        self.assertRaises(PostError, load_posts, _test_filename('encoding'),
                           _default_dict())
 
     def test_load_post_list_encoding(self):
-        post_list = load_posts('test/encoding/',
+        post_list = load_posts(_test_filename('encoding'),
                                _default_dict(encoding='UTF-8'))
         self.assertEqual(len(post_list), 2)
         sorted_list = sorted(post_list)
@@ -61,19 +61,17 @@ class TestSimpleLoad(unittest.TestCase):
                          u'\xd6\xe9\xe8\xe4\n')
 
     def test_load_posts_duplicate(self):
-        self.assertRaises(IOError, load_posts, 'test/duplicate/', _default_dict())
+        self.assertRaises(IOError, load_posts, _test_filename('duplicate'),
+                          _default_dict())
 
 class TestPost(unittest.TestCase):
+    def test_empty(self):
+        self.assertRaises(ValueError, Post)
 
     def test_simple(self):
-        sample_post = '''\
-        title: test
-        date: 2008-1-1
-        author: test author
-        encoding: ascii
-
-        test.'''
-        post = Post(_file(sample_post))
+        sample_post = ('title: test\ndate: 2008-1-1\nauthor: test author\n'
+                       'encoding: ascii\n\ntest.')
+        post = Post(content=sample_post)
         self.assertEqual(post.title, u'test')
         self.assertEqual(post.date, datetime.date(2008, 1, 1))
         self.assertEqual(post.author, u'test author')
@@ -81,13 +79,11 @@ class TestPost(unittest.TestCase):
         self.assertEqual(post.content, u'test.')
 
     def test_encoding(self):
-        sample_post = u'''\
-        title: Test UTF-8 \xdcTF-8 ?
-        author: Henry Pr\xeacheur <henry@precheur.org>
-        encoding: utf8
-
-        blah \xdcTF-8.'''.encode('utf8') # convert to str
-        post = Post(_file(sample_post))
+        sample_post = (u'title: Test UTF-8 \xdcTF-8 ?\n'
+                       u'author: Henry Pr\xeacheur <henry@precheur.org>\n'
+                       u'encoding: utf8\n\n'
+                       u'blah \xdcTF-8.').encode('utf8') # convert to str
+        post = Post(content=sample_post)
         self.assertEqual(post.title, u'Test UTF-8 \xdcTF-8 ?')
         self.assertEqual(post.author,
                          u'Henry Pr\xeacheur <henry@precheur.org>')
@@ -95,16 +91,14 @@ class TestPost(unittest.TestCase):
         self.assertEqual(post.content, u'blah \xdcTF-8.')
 
     def test_no_title(self):
-        sample = '''No title in this post'''
-        self.assertRaises(PostError, Post, _file(sample))
+        self.assertRaises(PostError, Post, content='No title in this post')
 
     def test_no_payload(self):
-        sample_post = 'title: no payload\ndate: 2008-1-1'
         try:
-            Post(_file(sample_post))
+            Post(content='title: no payload\ndate: 2008-1-1')
         except PostError, e:
             self.assertEqual(e.args,
-                             ('<unknown filename>: does not have content',))
+                             ('<unknown filename>: no content',))
         else:
             self.failUnless(False) # Should not be there
 
@@ -112,30 +106,30 @@ class TestPost(unittest.TestCase):
         sample_post = ('title: bad encoding\ndate: 2008-1-1\n'
                        'encoding: bad-encoding\n\ntest')
         try:
-            Post(_file(sample_post))
+            Post(content=sample_post)
         except PostError, e:
             self.assertEqual(e.args,
                              ('<unknown filename>: unknown encoding: '
                               'bad-encoding',))
         else:
             self.failUnless(False) # Should not be there
-        self.assertRaises(PostError, Post, _file(sample_post))
+        self.assertRaises(PostError, Post, content=sample_post)
 
     def test_empty_author(self):
-        post = Post(_file('title: test\n\ntest'))
+        post = Post(content='title: test\n\ntest')
         self.assertEqual(post.author, u'')
         self.assertEqual(post.author.name(), u'')
         self.assertEqual(post.author.email(), u'')
 
     def test_default_author(self):
-        post = Post(_file('title: test\n\ntest'),
+        post = Post(content='title: test\n\ntest',
                     default_author=u'Test <test@test.org>')
         self.assertEqual(post.author, u'Test <test@test.org>')
         self.assertEqual(post.author.name(), u'Test')
         self.assertEqual(post.author.email(), u'test@test.org')
 
     def test_author(self):
-        post = Post(_file('title:test\nauthor: Test <test@test.org>\n\ntest'))
+        post = Post(content='title:test\nauthor: Test <test@test.org>\n\ntest')
         self.assertEqual(post.author, u'Test <test@test.org>')
         self.assertEqual(post.author.name(), u'Test')
         self.assertEqual(post.author.email(), u'test@test.org')
@@ -143,7 +137,7 @@ class TestPost(unittest.TestCase):
     def test_bad_date(self):
         sample_post = 'title: bad encoding\ndate: 20 bad date 08-1-1\n\ntest'
         try:
-            Post(_file(sample_post))
+            Post(content=sample_post)
         except PostError, e:
             self.assertEqual(e.args,
                              ("<unknown filename>: Unable to parse date "
@@ -152,10 +146,15 @@ class TestPost(unittest.TestCase):
         else:
             self.failUnless(False) # Should not be there
 
+    def test_file_no_date(self):
+        p = Post(_test_filename('date/no_date.txt'))
+        self.assert_(p.date)
+        self.assert_(isinstance(p.date, datetime.date))
+
     def test_markdown(self):
-        sample_post = ('title: markdown\ndate: 2008-9-12\n\n'
+        sample_post = ('title: markdown\ndate: 2008-9-12\nmarkup: markdown\n\n'
                        '*boo*\n\n----\nblah')
-        post = Post(_file(sample_post), markup='markdown')
+        post = Post(content=sample_post)
         self.assertEqual(post.get_html(), (u'<p><em>boo</em></p>\n\n'
                                            '<hr>\n\n<p>blah</p>\n'))
         self.assertEqual(post.get_xhtml(), (u'<p><em>boo</em></p>\n\n'
@@ -163,48 +162,37 @@ class TestPost(unittest.TestCase):
 
     def test_html(self):
         sample_post = 'title: markdown\ndate: 2008-9-12\n\n<p>boo<br>blah</p>'
-        post = Post(_file(sample_post), markup='html')
+        post = Post(content=sample_post)
         self.assertEqual(post.get_html(), u'<p>boo<br>blah</p>')
         self.assertEqual(post.get_xhtml(), u'<p>boo<br />blah</p>')
 
-
-class TestTemplate(unittest.TestCase):
-
-    env = environment(os.path.dirname(__file__))
-
-    def test_renderstring(self):
-        template = self.env.\
-                from_string('Hello {{ string_template|renderstring }}!')
-        self.assertEqual(template.render(dict(string_template=('{{ foo }} '
-                                                               'world'),
-                                              foo='crazy')),
-                         u'Hello crazy world!')
-
-    def test_renderstring_empty(self):
-        template = self.env.\
-                from_string('Hello {{ string_template|renderstring }}!')
-        self.assertEqual(template.render(dict(string_template='',
-                                              foo='crazy')),
-                         u'Hello !')
+    def test_cmp(self):
+        file1 = 'title: 1\ndate: 2008-1-1\n\ntest'
+        post1 = Post(content=file1)
+        post2 = Post(content='title: 2\ndate: 2007-12-31\n\ntest')
+        self.assert_(post1 > post2)
+        self.assertNotEqual(post1, post2)
+        self.assertEqual(post1, post1)
+        self.assertEqual(post1, Post(content=file1))
+        self.assertEqual([post2, post1].index(Post(content=file1)), 1)
 
 
-class TestGeneration(unittest.TestCase):
-
-    env = environment(os.path.dirname(__file__))
-
+class TempDirMixin(object):
     def setUp(self):
         self.tempdir = tempfile.mkdtemp()
 
     def tearDown(self):
         shutil.rmtree(self.tempdir)
 
+
+class TestDate(TempDirMixin, unittest.TestCase):
     def test_date(self):
         filename = os.path.join(self.tempdir, 'set_date.html')
         # First test a message without any date defined
         def file_without_date():
             open(filename, 'w').write('title: Some title\n\nSome content')
         file_without_date()
-        command_date([filename, '2008-1-1'], None)
+        command_date([filename, '2008-1-1'])
         message = email.message_from_file(open(filename))
         self.assert_('date' in message)
         self.assertEqual(message['date'], str(datetime.date(2008, 1, 1)))
@@ -213,18 +201,32 @@ class TestGeneration(unittest.TestCase):
             open(filename, 'w').write('title: Some title\ndate: 2008-12-31\n'
                                       '\nSome content')
         file_with_date()
-        command_date([filename, '2008-1-1'], None)
+        command_date([filename, '2008-1-1'])
         message = email.message_from_file(open(filename))
         self.assert_('date' in message)
         self.assertEqual(message['date'], str(datetime.date(2008, 1, 1)))
         # Test aliases
         for alias in ('now', 'today', 'tomorrow', 'next_day'):
             file_without_date()
-            command_date([filename, alias], None)
+            command_date([filename, alias])
             message = email.message_from_file(open(filename))
             self.assert_('date' in message)
 
-    def test_format_date(self):
+    def test_date_empty(self):
+        filename = os.path.join(self.tempdir, 'empty_date.html')
+        open(filename, 'w').write('title: title\n\ncontent')
+        command_date([filename])
+        message = email.message_from_file(open(filename))
+        self.assert_('date' in message)
+        self.assert_(message['date'] >= str(datetime.date.today()))
+
+        open(filename, 'w').write('title: title\ndate: 2010-2-4\n\ncontent')
+        command_date([filename])
+        message = email.message_from_file(open(filename))
+        self.assert_('date' in message)
+        self.assert_(message['date'] >= str(datetime.date.today()))
+
+    def test_format(self):
         from weblog.date import _format_date
 
         self.assertEqual(_format_date(datetime.\
@@ -236,29 +238,29 @@ class TestGeneration(unittest.TestCase):
                          '2008-01-01')
         self.assertRaises(TypeError, _format_date, datetime.time())
 
-    def _test_publish(self, dirname):
-        options = Values(dict(source_dir=os.path.join(os.path.\
-                                                      dirname(__file__),
-                                                      'test', dirname),
+
+class TestPublish(TempDirMixin, unittest.TestCase):
+    def _test(self, dirname):
+        options = Values(dict(source_dir=_test_filename(dirname),
                               output_dir=self.tempdir,
                               configuration_file='config.py',
                               debug=False))
         command_publish(None, options)
 
-    def test_publish_empty(self):
-        self._test_publish('empty')
+    def test_empty(self):
+        self._test('empty')
 
-    def test_publish_encoding(self):
-        self._test_publish('encoding')
+    def test_encoding(self):
+        self._test('encoding')
 
-    def test_publish_full_url(self):
-        self._test_publish('full_url')
+    def test_full_url(self):
+        self._test('full_url')
 
-    def test_publish_simple(self):
-        self._test_publish('simple')
+    def test_simple(self):
+        self._test('simple')
 
-    def test_publish_template_encoding(self):
-        self._test_publish('template_encoding')
+    def test_template_encoding(self):
+        self._test('template_encoding')
 
 
 class TestConfiguration(unittest.TestCase):
@@ -288,6 +290,9 @@ class TestConfiguration(unittest.TestCase):
         with self.__config("encoding = 'latin-1'") as f:
             self.assertEqual(configuration.read(f.name)['encoding'], 'latin-1')
 
+    def test_non_existent(self):
+        self.assertRaises(IOError, configuration.read, '')
+
 
 class TestUrlParser(unittest.TestCase):
     def test_full_url_parser_attrs(self):
