@@ -1,5 +1,5 @@
 __author__ = u'Henry Pr\u00EAcheur <henry@precheur.org>'
-__version__ = '2.3'
+__version__ = '2.4'
 __license__ = 'ISCL'
 
 def main():
@@ -60,7 +60,7 @@ def main():
     elif command == 'publish':
         command_publish(args, options)
     elif command == 'date':
-        command_date(args, options)
+        command_date(args)
 
 def test_main():
     import doctest
