@@ -29,18 +29,18 @@ def _format_date(date):
                         (datetime.date.__name__, datetime.datetime.__name__,
                          date.__class__.__name__))
 
-def command_date(args, options):
+def command_date(args):
     '''
     Execute the 'date' command, which set the date to the specified filename.
     The command need at least one parameter. The remaining parameters are the
     date to be set in the file.
 
-    >>> command_date(None, None) # doctest: +ELLIPSIS
+    >>> command_date(None) # doctest: +ELLIPSIS
     Traceback (most recent call last):
     ...
     SystemExit: No file specified:
     ...
-    >>> command_date(['/dev/null', '2008-1000-10'], None)
+    >>> command_date(['/dev/null', '2008-1000-10'])
     Traceback (most recent call last):
     ...
     SystemExit: Unable to parse date '2008-1000-10'
