@@ -1,9 +1,9 @@
 import re
-import email
 import codecs
 import locale
 import logging
 import datetime
+from email import message_from_file, message_from_string
 from os import stat, path
 from urllib import quote
 
@@ -19,9 +19,6 @@ except ImportError:
 from html_to_xhtml import html_to_xhtml
 
 class PostError(Exception):
-    '''
-    Error in post file
-    '''
     def __init__(self, filename, message):
         Exception.__init__(self, '%s: %s' % (filename, message))
 
@@ -71,105 +68,98 @@ class Author(unicode):
             return u''
 
 class Post(object):
+    def _error(self, *args, **kwargs):
+        return PostError(self.source_filename, *args, **kwargs)
 
-    def __init__(self, f,
-                 markup=None,
+    def __init__(self, filename=None, content=None,
                  default_encoding=u'UTF-8',
                  default_author=u'',
                  filesystem_encoding=locale.getpreferredencoding()):
-        default_author = Author(default_author)
-        if isinstance(f, basestring):
-            self._filename = f
-            input_file = open(f)
+        self._filename = filename
+        if content:
+            post_file = message_from_string(content)
+        elif filename:
+            post_file = message_from_file(open(filename))
         else:
-            self._filename = None
-            input_file = f
-        post_file = email.message_from_file(input_file)
+            raise ValueError('filename or content are required.')
 
-        if 'title' not in post_file:
-            raise PostError(self.get_filename(), 'No title')
+        headers = dict(post_file.items())
+        content = post_file.get_payload()
 
         # Get the file's encoding
-        self.encoding = unicode(post_file.get('encoding') or
-                                default_encoding)
+        self.encoding = unicode(headers.pop('encoding', default_encoding))
         try:
             codecs.lookup(self.encoding)
         except LookupError, e:
-            raise PostError(self.get_filename(), str(e))
+            raise self._error(str(e))
 
-        # Copy all field "into the object" and convert string to unicode.
+        if not content:
+            raise self._error('no content')
         try:
-            for key, value in post_file.items():
-                self.__dict__[key.encode('ascii').lower()] = \
-                        unicode(value, self.encoding)
-        except UnicodeDecodeError, e:
-            raise PostError(self.get_filename(), "for key '%s': %s" % (key, e))
-
-        if not hasattr(self, 'author'):
-            self.author = default_author
-        else:
-            self.author = Author(self.author)
-
-        # Handle the date. If no date was specified use the file's modification
-        # time.
-        if not hasattr(self, 'date'):
-            if not self._filename:
-                self.date = None
-            else:
-                # Get the date from file's mtime and issue a warning
-                mtime = stat(self._filename).st_mtime
-                self.date = datetime.datetime.fromtimestamp(mtime)
-                logging.warning("No date defined in '%s', using the file's "
-                                "last modification time instead." % \
-                                self._filename)
-        else:
-            try:
-                self.date = self.parse_date(self.date)
-            except ValueError, e:
-                raise PostError(self.get_filename(), str(e))
-
-        if not hasattr(self, 'slug'):
-            try:
-                self.slug = self.title.\
-                        encode(filesystem_encoding, 'replace').\
-                        replace(' ', '_')
-            except UnicodeDecodeError, e:
-                raise PostError(self.get_filename(), 'Bad encoding in title')
-        self.slug = self.slug.replace('/\\', '') # Remove '/' and '\'.
-
-        if not post_file.get_payload():
-            raise PostError(self.get_filename(), 'does not have content')
-
-        try:
-            self.content = unicode(post_file.get_payload(), self.encoding)
+            self.content = unicode(content, self.encoding)
         except UnicodeDecodeError, e:
             # find error line number
             for line_number, line in enumerate(post_file.as_string().\
                                                splitlines()):
                 try:
-                    line.decode('ascii' if self.encoding == 'raw'
-                                else self.encoding)
+                    line.decode(self.encoding)
                 except UnicodeDecodeError, e:
                     break
             # line_number starts at 0, real line number == line_number + 1
-            raise PostError(self.get_filename(),
-                            'Bad encoding in content line %d, %s' % \
-                            (line_number + 1, e))
+            raise self._error('Bad encoding in content line %d, %s' %
+                              (line_number + 1, e))
+        del post_file # XXX
 
-        if not markup: # Determine type via file extension
-            if self._filename and self._filename.endswith('.txt'):
-                self._markup = 'markdown'
-            elif self._filename and self._filename.endswith('.html'):
-                self._markup = 'html'
-            elif not self._filename:
-                self._markup = 'html'
-            else:
-                logging.warning("Unable to determine '%s' type, falling "
-                                "back to HTML" % self._filename)
-                self._markup = 'html'
+        # Copy all field "into the object" and convert string to unicode.
+        for key, value in headers.iteritems():
+            if not key.islower():
+                logging.warning('%r will be renamed to %r' % (key,
+                                                              key.lower()))
+            try:
+                key = key.encode('ascii').lower()
+            except UnicodeDecodeError, e:
+                raise self._error("Post attributes can't contain "
+                                  'non-ascii characters: %r' % key)
+            if hasattr(self, key):
+                raise self._error('%s is reserved' % key)
+            try:
+                self.__dict__[key] = unicode(value, self.encoding)
+            except UnicodeDecodeError, e:
+                raise self._error("Unable to decode attribute's %s value" %
+                                  key)
+
+        # XXX title might not be required in future versions
+        if not hasattr(self, 'title'):
+            raise self._error('No title')
+
+        if not hasattr(self, 'author'):
+            self.author = Author(default_author)
         else:
-            assert markup in ('markdown', 'html')
-            self._markup = markup
+            self.author = Author(self.author)
+
+        # If no date was specified use the file's modification time.
+        if not hasattr(self, 'date'):
+            if not self._filename:
+                self.date = None
+            else:
+                # Get the date from the file's mtime and issue a warning
+                self.date = datetime.datetime.fromtimestamp(self.mtime)
+                logging.warning("No date defined in '%s', using the file's "
+                                "last modification time instead." %
+                                self._filename)
+        else:
+            try:
+                self.date = self.parse_date(self.date)
+            except ValueError, e:
+                raise self._error(str(e))
+
+        if not hasattr(self, 'slug'):
+            try:
+                self.slug = self.title.encode(filesystem_encoding, 'replace')
+            except UnicodeDecodeError, e:
+                raise self._error('Bad encoding in title')
+        for x in '/\\ ':
+            self.slug = self.slug.replace(x, '_')
 
         # Transform the 'files' field into a list of string
         if hasattr(self, 'files'):
@@ -180,8 +170,7 @@ class Post(object):
     def directories(self):
         r'''Return the list of directories where the post is stored.
 
-            >>> from StringIO import StringIO
-            >>> p = Post(StringIO('title: test\ndate: 2009-9-25\n\ntest'))
+            >>> p = Post(content='title: test\ndate: 2009-9-25\n\ntest')
             >>> p.directories()
             ('2009', '9', '25')
         '''
@@ -192,8 +181,7 @@ class Post(object):
     def filename(self):
         r'''Return the filename where to post should be stored.
 
-            >>> from StringIO import StringIO
-            >>> p = Post(StringIO('title: test\ndate: 2009-9-25\n\ntest'))
+            >>> p = Post(content='title: test\ndate: 2009-9-25\n\ntest')
             >>> p.filename()
             '2009/9/25/test.html'
         '''
@@ -202,21 +190,26 @@ class Post(object):
     def url(self):
         r'''Returns url of the post.
 
-            >>> file_content = """title: test
+            >>> content = """title: test
             ... date: 2008-1-1
             ...
             ... test"""
-            >>> from StringIO import StringIO
-            >>> Post(StringIO(file_content)).url()
+            >>> Post(content=content).url()
             '2008/1/1/test.html'
 
         The url is URL-quoted:
 
-            >>> Post(StringIO('title: @!%\ndate: 2009-10-1\n\ntest')).url()
+            >>> Post(content='title: @!%\ndate: 2009-10-1\n\ntest').url()
             '2009/10/1/%40%21%25.html'
         '''
         return '/'.join(self.directories() + (quote(self.slug) + '.html',))
 
+    @property
+    def mtime(self):
+        if not hasattr(self, '_mtime'):
+            self._mtime = stat(self._filename).st_mtime
+        return self._mtime
+
     _DATE_FORMAT_LIST = ('%Y-%m-%d', '%y-%m-%d')
 
     _DATETIME_FORMAT_LIST = \
@@ -256,26 +249,40 @@ class Post(object):
                          '(Use YYYY-MM-DD [[HH:MM]:SS] format)' %
                          (date_))
 
-    def get_filename(self):
+    @property
+    def source_filename(self):
         if not self._filename:
             return '<unknown filename>'
         else:
             return self._filename
 
+    def _markup(self):
+        if hasattr(self, 'markup'):
+            return self.markup
+        # Determine type via file extension
+        if self._filename and self._filename.endswith('.txt'):
+            return 'markdown'
+        elif not self._filename or self._filename.endswith('.html'):
+            return 'html'
+        else:
+            logging.warning('Unable to determine %r markup, falling '
+                            'back to HTML' % self._filename)
+            return 'html'
+
     def get_html(self):
-        if self._markup == 'markdown':
+        if self._markup() == 'markdown':
             return markdown(self.content, html4tags=True,
                             extras={'demote-headers': 2})
-        elif self._markup == 'html':
+        elif self._markup() == 'html':
             return self.content
         else:
             assert False, "Unknown type for %r" % self
 
     def get_xhtml(self):
-        if self._markup == 'markdown':
+        if self._markup() == 'markdown':
             return markdown(self.content, html4tags=False,
                             extras={'demote-headers': 2})
-        elif self._markup == 'html':
+        elif self._markup() == 'html':
             return html_to_xhtml(self.content)
         else:
             assert False, "Unknown type for %r" % self
@@ -293,25 +300,10 @@ class Post(object):
         return self.date.day
 
     def __cmp__(self, other):
-        '''
-        >>> file1 = "title: 1\\ndate: 2008-1-1\\n\\ntest"
-        >>> file2 = "title: 2\\ndate: 2007-12-31\\n\\ntest"
-        >>> from StringIO import StringIO
-        >>> Post(StringIO(file1)) > Post(StringIO(file2))
-        True
-        >>> Post(StringIO(file1)) == Post(StringIO(file2))
-        False
-        >>> Post(StringIO(file1)) == Post(StringIO(file1))
-        True
-        >>> l = [Post(StringIO(file2)), Post(StringIO(file1))]
-        >>> l.index(Post(StringIO(file1)))
-        1
-        '''
-        return cmp(unicode(self.date) + self.title,
-                   unicode(other.date) + other.title)
+        return cmp(str(self.date) + self.title, str(other.date) + other.title)
 
     def __hash__(self):
-        return hash(str(self.date) + self.title)
+        return hash(self.filename())
 
     def __repr__(self):
         return '<%s(%r, %r)>' % (self.__class__.__name__, self.title, self.date)
