@@ -5,18 +5,11 @@ import codecs
 from shutil import copy
 
 import weblog
+import template
 import configuration
-from template import environment
 from html_full_url import html_full_url
 from post import Post, PostError
 
-def _write_file(filename, encoding, content):
-    f = codecs.open(filename, mode='w', encoding=encoding)
-    try:
-        f.write(content)
-    finally:
-        f.close()
-
 def _check_duplicated(p, posts):
     if p in posts:
         logging.debug('%r is duplicated', p)
@@ -25,7 +18,7 @@ def _check_duplicated(p, posts):
                 break
         raise IOError('%s, there is already a post '
                       'with this title and date (%s)' % \
-                      (p.get_filename(), duplicated_post.get_filename()))
+                      (p.source_filename, duplicated_post.source_filename))
 
 def load_posts(source_dir, config):
     posts = set()
@@ -48,13 +41,9 @@ def load_posts(source_dir, config):
                 logging.debug('Ignoring %s', filename)
     return posts
 
-def generate_post_html(post_list, output_dir, post_tmpl, config):
+def generate_post_html(post_list, writer, config):
     for post in post_list:
         logging.debug('Generating HTML file for %r', post)
-        dir = os.path.join(output_dir, *post.directories())
-        if not os.path.isdir(dir):
-            logging.debug('Creating directory %s', dir)
-            os.makedirs(dir)
         top_dir = '../../../'
         params = dict(config)
         params.update(dict(title=post.title,
@@ -63,9 +52,8 @@ def generate_post_html(post_list, output_dir, post_tmpl, config):
                            content=html_full_url(top_dir,
                                                  post.get_html()),
                            top_dir=top_dir))
-        _write_file(os.path.join(output_dir, post.filename()),
-                    config.get('encoding'),
-                    post_tmpl.render(**params))
+        writer.write('post.html.tmpl', post.filename(), timestamp=post.mtime,
+                     **params)
 
 def command_publish(args, options):
     source_dir = options.source_dir
@@ -79,51 +67,47 @@ def command_publish(args, options):
 
     source_dir = source_dir or config.get('source_dir', '.')
     output_dir = output_dir or config.get('output_dir', 'output')
-    if 'filesystem_encoding' in config:
-        Post._FILESYSTEM_ENCODING = config['filesystem_encoding']
 
     if not os.path.exists(output_dir):
         os.mkdir(output_dir)
 
-    env = environment(source_dir)
-
     try:
         post_list = sorted(load_posts(source_dir, config), reverse=True)
     except (IOError, PostError), e:
         logging.error('Error while loading post files.')
         raise SystemExit(e)
 
+    writer = template.Writer(source_dir, output_dir, config.get('encoding'))
+
     def generate_all():
+        max_mtime = max(p.mtime for p in post_list) if post_list else 0
+
         logging.debug('Generating archives page')
-        archives_template = env.get_template('archives.html.tmpl')
-        _write_file(os.path.join(output_dir, 'archives.html'),
-                    config.get('encoding'),
-                    archives_template.render(posts=post_list,
-                                             **config))
+        writer.write('archives.html.tmpl', 'archives.html', posts=post_list,
+                     timestamp=max_mtime, **config)
 
         logging.debug('Generating main page')
-        index_template = env.get_template('index.html.tmpl')
-        _write_file(os.path.join(output_dir, 'index.html'),
-                    config.get('encoding'),
-                    index_template.render(posts=post_list,
-                                          **config))
+        writer.write('index.html.tmpl', 'index.html', posts=post_list,
+                     timestamp=max_mtime, **config)
 
         logging.debug('Generating HTML posts files')
-        post_tmpl = env.get_template('post.html.tmpl')
-        generate_post_html(post_list, output_dir, post_tmpl, config)
+        generate_post_html(post_list, writer, config)
         # Copy all 'attached' files
         for post in post_list:
             for filename in post.files:
-                destination = os.path.join(output_dir, filename)
+                src = os.path.join(source_dir, filename)
+                dst = os.path.join(output_dir, filename)
+
                 # Create the destination directory if it does not exist
-                destination_dir = os.path.dirname(destination)
-                # isdir returns False if the passed file does not exist
+                destination_dir = os.path.dirname(dst)
                 if not os.path.isdir(destination_dir):
                     os.makedirs(destination_dir)
-                copy(os.path.join(source_dir, filename), destination)
+
+                if (not os.path.isfile(dst) or
+                    os.stat(src).st_mtime >= os.stat(dst).st_mtime):
+                    copy(os.path.join(source_dir, filename), dst)
 
         # Generate Atom feed
-        template = env.get_template('feed.atom.tmpl')
 
         # Last time the feed was updated
         posts = post_list[:config.get('feed_limit', 10)]
@@ -137,13 +121,11 @@ def command_publish(args, options):
         else:
             feed_updated = datetime.datetime.utcnow()
 
-        atom_file = codecs.open(os.path.join(output_dir, 'feed.atom'), 'w',
-                                encoding='utf8')
-        atom_file.write(template.render(posts=posts,
-                                        feed_updated=feed_updated,
-                                        weblog_version=weblog.__version__,
-                                        **config))
-        atom_file.close()
+        writer.write('feed.atom.tmpl', 'feed.atom', encoding='utf-8',
+                     posts=posts, feed_updated=feed_updated,
+                     weblog_version=weblog.__version__,
+                     timestamp=max(x.mtime for x in posts) if posts else 0,
+                     title=config.get('title', None))
 
         for f in config.get('extra_files', tuple()):
             copy(os.path.join(source_dir, f), output_dir)
