@@ -9,9 +9,9 @@ class UTF8HTMLParser(HTMLParser):
         >>> parser.feed("<p>Hello <a href='crazy'>world</a></p>")
         >>> parser.get_value()
         u'<p>Hello <a href="crazy">world</a></p>'
-        >>> parser.feed('<p>Another sentence.</p>')
+        >>> parser.feed('<p>Another<br/>sentence.</p>')
         >>> parser.get_value()
-        u'<p>Hello <a href="crazy">world</a></p><p>Another sentence.</p>'
+        u'<p>Hello <a href="crazy">world</a></p><p>Another<br>sentence.</p>'
 
     `reset()` resets the parser::
 
